/**
  ******************************************************************************
  * @file main.c
  * @brief This file contains the firmware main function.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.0.0
  * @date 10/13/2008
  ******************************************************************************
  *
  */


// Includes ------------------------------------------------------------------

#include "stm8s.h"
#include "swuart.h"


  // * @addtogroup SW_UART
  
// Private typedef -----------------------------------------------------------

// Private define ------------------------------------------------------------

// Evalboard I/Os configuration 
#define LEDS_PORT (GPIOH)
#define ALL_LEDs 	(GPIO_PIN_3 | GPIO_PIN_2 | GPIO_PIN_1 | GPIO_PIN_0) // LEDs mask (EVAL board)

// Private macro -------------------------------------------------------------

#define switch_all_LEDs_on		{ GPIO_WriteHigh(LEDS_PORT, ALL_LEDs); }	//LEDs control
#define switch_all_LEDs_off	{ GPIO_WriteLow(LEDS_PORT, ALL_LEDs); }

// Private variables ---------------------------------------------------------

	u8 tx_byte;								// transmitted byte
	
// Private function prototypes -----------------------------------------------
/**
  * @brief Programable loop delay
  * @par Parameters:
  * wt: number of loops
  * @retval None
  */
void delay_loop(u16 wt) {
	while(wt--);
}
// ---------------------------------------------------------------------------
/**
  * @brief Simple example of use swuart procedures
  * @par Parameters:
  * None
  * @retval None
  */
void main(void) {
	u8 buff, sts;
#ifdef TO_HSE
	ErrorStatus val;
#endif
		
// 								*** GPIO INIT ***
	GPIO_Init(LEDS_PORT, ALL_LEDs, GPIO_MODE_OUT_PP_LOW_FAST);
		
// 								*** CLK INIT ***
#ifdef TO_HSE
	CLK_SYSCLKConfig(CLK_PRESCALER_CPUDIV1);	// set the highest HSE speed
	val= CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO,CLK_SOURCE_HSE,DISABLE,CLK_CURRENTCLOCKSTATE_ENABLE);  
#else
	CLK_SYSCLKConfig(CLK_PRESCALER_HSIDIV1);	// set the highest HSI speed
#endif

	uart_init();							// init pins and variables of SW UART
	uart_receive_enable;					// enable receive process
	
	enableInterrupts();					// enable interrupts
	
	tx_byte= 0xAA;							// first byte to send
	
// 								*** MAIN LOOP ***
	for(;;) {
		sts= uart_send(tx_byte);							// send a byte
		while(test_status(transmit_in_progress) != 0);// wait for its transmition complete
		switch_all_LEDs_on;									// LEDs on
		delay_loop(100);										// dummy loop
		sts= uart_read(&buff);								// read receive buffer
		if(sts != receive_buffer_full	|| buff != tx_byte) // check content of the status & data registers
			switch_all_LEDs_off;								// LEDs off if the data byte isn't correctly received
		delay_loop((u16)(50000));							// let the LEDs indication be observed
		++tx_byte;												// data byte change
	};
}

/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
  // Put enter your own code to manage an assert error 
}

